#include "cmd_processer_server.h"
#include "mqtt_request.h"

namespace offcn
{
	static const std::string kGlobalTopic = "kcp_server_mqtt";
	static const std::string kLocalID = "kcp_server_mqtt";

	CmdProcesserServer::CmdProcesserServer()
		:mqtt_wrapper_(NULL)
	{
		mqtt_wrapper_ = new offcn::MqttWrapper(this);
	}
	CmdProcesserServer::~CmdProcesserServer()
	{
		if (mqtt_wrapper_)
		{
			delete mqtt_wrapper_;
			mqtt_wrapper_ = NULL;
		}
	}

	bool CmdProcesserServer::Connect()
	{
		return mqtt_wrapper_->ConnectMqttServer(kLocalID);
	}
	bool CmdProcesserServer::SubscribeTopic()
	{
		return mqtt_wrapper_->SubscribeTopic(kGlobalTopic);
	}

	/**
	 * CmdObserver
	 */
	void CmdProcesserServer::OnServerConnectSuccess()
	{
		printf("connect mqtt server success!\n");

		SubscribeTopic();
	}
	void CmdProcesserServer::OnServerConnectFailure()
	{
		printf("connect mqtt server failure!\n");
	}
	void CmdProcesserServer::OnServerSubscribeSuccess()
	{
		printf("subscribe topic success!\n");
	}
	void CmdProcesserServer::OnServerSubscribeFailure()
	{
		printf("subscribe topic failure!\n");
	}
	void CmdProcesserServer::OnServerMessageArrived(std::string topic, std::string message)
	{
		printf("\nreceive message : topic = %s, context : %s\n", topic.c_str(), message.c_str());

		std::string path;
		ReqValue	values;
		if (!MqttRequest::ParseRequest(message, path, values))
		{
			printf("Error : Parse request error\n");
			return;
		}

		std::string fid = values["fid"];
		std::string pull_type;
		if (path.compare("join") == 0)
		{
			pull_type = values["pull_type"];
			OnRequstJoin(fid, pull_type);
		}
	}

	void CmdProcesserServer::OnRequstJoin(std::string fid, std::string pull_type)
	{
		if (client_lists_.find(fid) != client_lists_.end())
		{
			printf("Waring : fid = %s already exist\n", fid.c_str());
			return;
		}

		std::string dest_id;

		std::map<std::string, std::string>::iterator itor;
		for (itor = client_lists_.begin(); itor != client_lists_.end(); itor++)
		{
			if (itor->second.compare("rtmp") == 0)
			{
				dest_id = itor->first;
				break;
			}
		}

		std::string response;

		if (itor != client_lists_.end())
		{
			client_lists_.insert(std::pair<std::string, std::string>(fid, "kcp"));
			response = MqttRequest::JoinResponse(dest_id, "success");
		}
		else
		{
			client_lists_.insert(std::pair<std::string, std::string>(fid, "rtmp"));
			response = MqttRequest::JoinResponse("", "success");
		}

		mqtt_wrapper_->SendRequest(fid, response);

		printf("\n------------------------------------\n");
		for (itor = client_lists_.begin(); itor != client_lists_.end(); itor++)
		{
			printf("id = %s, pull type = %s\n", itor->first.c_str(), itor->second.c_str());
		}
		printf("------------------------------------\n");
	}
}