#include "cmd_processer_client.h"
#include "mqtt_request.h"

namespace offcn
{
	static const std::string kGlobalTopic = "kcp_server_mqtt";
	static const std::string kLocalID = "mqtt_client";
	static const std::string kDstTopic = kLocalID;

	CmdProcesserClient::CmdProcesserClient()
		:mqtt_wrapper_(NULL)
	{
		local_id_ = kLocalID;

		mqtt_wrapper_ = new offcn::MqttWrapper(this);
	}
	CmdProcesserClient::~CmdProcesserClient()
	{
		if (mqtt_wrapper_)
		{
			delete mqtt_wrapper_;
			mqtt_wrapper_ = NULL;
		}
	}

	void CmdProcesserClient::SetLocalID(std::string id)
	{
		local_id_ = id;
	}
	bool CmdProcesserClient::Connect()
	{
		return mqtt_wrapper_->ConnectMqttServer(local_id_);
	}
	bool CmdProcesserClient::SubscribeTopic()
	{
		return mqtt_wrapper_->SubscribeTopic(local_id_);
	}
	bool CmdProcesserClient::SendLocalCandidate(std::string tid, std::string type, std::string candidate)
	{
		std::string req = MqttRequest::Candidate(local_id_, tid, type, candidate);
		return mqtt_wrapper_->SendRequest(tid, req);
	}

	/**
	 * CmdObserver
	 */
	void CmdProcesserClient::OnServerConnectSuccess()
	{
		printf("connect mqtt server success!\n");
		SubscribeTopic();
	}
	void CmdProcesserClient::OnServerConnectFailure()
	{
		printf("connect mqtt server failure!\n");
	}
	void CmdProcesserClient::OnServerSubscribeSuccess()
	{
		printf("subscribe topic success!\n");
		
		std::string topic = kGlobalTopic;
		std::string req = MqttRequest::JoinRequest(local_id_);;

		mqtt_wrapper_->SendRequest(topic, req);
	}
	void CmdProcesserClient::OnServerSubscribeFailure()
	{
		printf("subscribe topic failure!\n");
	}
	void CmdProcesserClient::OnServerMessageArrived(std::string topic, std::string message)
	{
		std::string path;
		ReqValue	values;
		if (!MqttRequest::ParseResponse(message, path, values))
		{
			printf("Error : Parse request error\n");
			return;
		}

		if (path.compare("join") == 0)
		{
			std::string ids = values["ids"];
			std::string result = values["result"];
			if (ids.length() > 0)
			{
				publisher_list_.insert(std::pair<std::string, std::string>(ids, ids));
			}
		}
	}
}