#include "mqtt_request.h"
#include "json/json.h"

namespace offcn
{
	/*
	{
		"path": "join",
		"fid" : "xxxid",
		"data" :
					{
						"pull_type" : "null"
					}
	}
	*/
	std::string MqttRequest::JoinRequest(std::string local_id)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "join";
		root["fid"] = local_id;

		Json::Value data;
		data["pull_type"] = "null";

		root["data"] = data;
		
		return Json::writeString(wBuilder, root);
	}
	/*
	{
		"path": "update",
		"fid" : "xxxid",
		"data" :
					{
						"pull_type" : "rtmp" or "null" or "kcp"
					}
	}
	*/
	std::string MqttRequest::UpdateRequest(std::string local_id, std::string pull_type)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "update";
		root["fid"] = local_id;

		Json::Value data;
		data["pull_type"] = pull_type;

		root["data"] = data;

		return Json::writeString(wBuilder, root);
	}
	/*
	{
		"path": "offer/answer",
		"fid" : "xxxid",
		"tid" : "remoteid",
		"candidate" : ""
	}
	*/
	std::string MqttRequest::Candidate(std::string local_id, std::string remote_id, std::string type, std::string candidate)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = type;
		root["fid"] = local_id;
		root["tid"] = remote_id;
		root["candidate"] = candidate;

		return Json::writeString(wBuilder, root);
	}

	static bool GetJsonValue(std::string &response, Json::Value &root)
	{
		Json::CharReaderBuilder readerBuilder;
		std::unique_ptr<Json::CharReader> const reader(readerBuilder.newCharReader());
		const char *start_pos = response.c_str();
		std::string err;
		if (!reader->parse(start_pos, start_pos + response.length(), &root, &err))
		{
			return false;
		}

		return true;
	}

	bool MqttRequest::ParseRequest(std::string &req, std::string &path, ReqValue &values)
	{
		Json::Value root;

		if (!GetJsonValue(req, root)) return false;

		if (!root["path"] || !root["fid"]) return false;

		path = root["path"].asString();
		std::string fid = root["fid"].asString();

		if (path.compare("join") == 0 ||
			path.compare("update") == 0)
		{
			values["fid"] = fid;
			values["pull_type"] = root["pull_type"].asString();
		}
		else if (path.compare("offer") == 0 ||
			path.compare("answer") == 0)
		{
			values["fid"] = fid;
			values["tid"] = root["tid"].asString();
			values["candidate"] = root["candidate"].asString();
		}

		return true;
	}
	bool MqttRequest::ParseResponse(std::string &response, std::string &path, ReqValue &values)
	{
		Json::Value root;

		if (!GetJsonValue(response, root)) return false;

		if (!root["path"] || !root["fid"]) return false;

		path = root["path"].asString();
		std::string fid = root["fid"].asString();
		if (path.compare("join") == 0)
		{
			values["ids"] = root["ids"].asString();
			values["result"] = root["result"].asString();
		}
		return true;
	}

	std::string MqttRequest::JoinResponse(std::string ids, std::string result)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "join";
		root["ids"] = ids;
		root["result"] = result;

		return Json::writeString(wBuilder, root);
	}
	std::string MqttRequest::UpdateResponse(std::string result)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "update";
		root["result"] = result;

		return Json::writeString(wBuilder, root);
	}
}